/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.clearquest;

import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class InputSourceBeanHandler {

	private InputSourceBean inputSourceBean;
	private Date lastOperationExecutionDate;
	private static final String S4Q_LAST_EXTRACTION = "S4Q_LAST_EXTRACTION";
	private static final String CQ_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
	private static final String ATTRIBUTE_REFLIST = "reflist";
	private static final String SPLIT_CHARATER = ";";

	public InputSourceBeanHandler(String entryName,
			Date lastOperationExecutionDate) {
		inputSourceBean = new InputSourceBean(entryName);
		this.lastOperationExecutionDate = lastOperationExecutionDate;
	}

	public void setEntry(String[] attributeValues) {
		int lenght = 0;
		if (attributeValues != null)
			lenght = attributeValues.length;

		switch (lenght) {
		case 1:
			/*
			 * id -> <attribute name="id">
			 */
			inputSourceBean.setStandardElement(attributeValues[0]);
			break;
		case 2:
			/*
			 * id;yes -> <attribute name="id" reflist="yes">
			 */
			AttributeElement elementWithRefList = new AttributeElement();
			elementWithRefList.setAttribute(InputSourceBean.ATTRIBUTE_NAME,
					attributeValues[0]);
			elementWithRefList.setAttribute(ATTRIBUTE_REFLIST,
					attributeValues[1]);
			inputSourceBean.setElement(elementWithRefList);
			break;
		case 3:
			/*
			 * id;GT;ARG1 -> <attribute name="id"> <filter operation="GT"
			 * value=ARG1/> </attribute>
			 */
			AttributeElement elementWithFilter = new AttributeElement();
			elementWithFilter.setAttribute(InputSourceBean.ATTRIBUTE_NAME,
					attributeValues[0]);
			elementWithFilter.setFilter(attributeValues[1],
					transformOperationFilterValue(attributeValues[2]));
			inputSourceBean.setElement(elementWithFilter);
			break;
		case 4:
			/*
			 * id;yes;GT;ARG1 -> <attribute name="id" reflist="yes"> <filter
			 * operation="GT" value=ARG1/> </attribute>
			 */
			AttributeElement elementWithRefListAndFilter = new AttributeElement();
			elementWithRefListAndFilter.setAttribute(
					InputSourceBean.ATTRIBUTE_NAME, attributeValues[0]);
			elementWithRefListAndFilter.setAttribute(ATTRIBUTE_REFLIST,
					attributeValues[1]);
			elementWithRefListAndFilter.setFilter(attributeValues[2],
					transformOperationFilterValue(attributeValues[3]));
			inputSourceBean.setElement(elementWithRefListAndFilter);
			break;
		default:
			break;
		}
	}

	public String getInputSourceBeanString() {
		return inputSourceBean.toString();
	}

	public void setInputSourceBean(List<String> readOperationParameterValueList) {
		for (String string : readOperationParameterValueList) {
			setEntry(string.split(SPLIT_CHARATER));
		}
	}

	private String transformOperationFilterValue(String value) {
		String toReturn = value;
		if (value != null && value.equals(S4Q_LAST_EXTRACTION)) {
			if (lastOperationExecutionDate != null) {
				Format formatter = new SimpleDateFormat(CQ_DATE_FORMAT);
				toReturn = formatter.format(lastOperationExecutionDate);
			} else {
				toReturn = null;
			}
		}
		return toReturn;
	}
}
